# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.cloud import get_custom_clouds
from azure.cli.core.decorators import Completer

from azure.cli.command_modules.cloud.custom import get_clouds


@Completer
def get_cloud_name_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
    return [c.name for c in get_clouds(cmd.cli_ctx)]


@Completer
def get_custom_cloud_name_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
    return [c.name for c in get_custom_clouds(cmd.cli_ctx)]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def set_metrics(client, retention, hour=None, minute=None, api=None, timeout=None):
    for s in client:
        s.set_metrics(retention, hour, minute, api, timeout)


def get_metrics(client, interval='both', timeout=None):
    results = {}
    for s in client:
        results[s.name] = s.get_metrics(interval, timeout)
    return results



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=too-many-lines
# pylint: disable=inconsistent-return-statements
# pylint: disable=protected-access
# pylint: disable=too-many-locals

import argparse


class AlertAddEncryption(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        action = self.get_action(values, option_string)
        super(AlertAddEncryption, self).__call__(parser, namespace, action, option_string)

    def get_action(self, values, option_string):  # pylint: disable=no-self-use
        from azure.mgmt.eventhub.v2021_11_01.models import KeyVaultProperties
        from azure.mgmt.eventhub.v2021_11_01.models import UserAssignedIdentityProperties
        from azure.cli.core.azclierror import InvalidArgumentValueError
        from azure.cli.core import CLIError

        keyVaultObject = KeyVaultProperties()

        for (k, v) in (x.split('=', 1) for x in values):
            if k == 'key-name':
                keyVaultObject.key_name = v
            elif k == 'key-vault-uri':
                keyVaultObject.key_vault_uri = v
                if keyVaultObject.key_vault_uri.endswith('/'):
                    keyVaultObject.key_vault_uri = keyVaultObject.key_vault_uri[:-1]
            elif k == 'key-version':
                keyVaultObject.key_version = v
            elif k == 'user-assigned-identity':
                keyVaultObject.identity = UserAssignedIdentityProperties()
                keyVaultObject.identity.user_assigned_identity = v
                if keyVaultObject.identity.user_assigned_identity.endswith('/'):
                    keyVaultObject.identity.user_assigned_identity = keyVaultObject.identity.user_assigned_identity[:-1]
            else:
                raise InvalidArgumentValueError("Invalid Argument for:'{}' Only allowed arguments are 'key-name, key-vault-uri, key-version and user-assigned-identity'".format(option_string))

        if (keyVaultObject.key_name is None) or (keyVaultObject.key_vault_uri is None):
            raise CLIError('key-name and key-vault-uri are mandatory properties')

        if keyVaultObject.key_version is None:
            keyVaultObject.key_version = ''

        return keyVaultObject



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long


class ErrorClass:
    error_title = ""
    error_message = ""

    def __init__(self, title, message):
        self.error_title = title
        self.error_message = message

    def get_error_message(self, additional_message=None):
        if additional_message:
            return "An error occurred: {}\n{}\n{}".format(self.error_title, self.error_message, additional_message)

        return "An error occurred: {}\n{}".format(self.error_title, self.error_message)

    def set_error_message(self, message):
        return ErrorClass(self.error_title, message)

    def append_error_message(self, message):
        return self.set_error_message(self.error_message + "\n" + message)

    def format_error_message(self, *args):
        return self.set_error_message(self.error_message.format(*args))


# DOCKER ERRORS
DOCKER_COMMAND_ERROR = ErrorClass(
    "DOCKER_COMMAND_ERROR",
    "Please verify if Docker client is installed and running."
)


DOCKER_DAEMON_ERROR = ErrorClass(
    "DOCKER_DAEMON_ERROR",
    "Please verify if Docker daemon is running, or try restarting it."
)


DOCKER_VERSION_ERROR = ErrorClass(
    "DOCKER_VERSION_ERROR",
    "An error occurred while retrieving Docker version. Please try verifying it manually."
)


DOCKER_PULL_ERROR = ErrorClass(
    "DOCKER_PULL_ERROR",
    "An error occurred while pulling a sample image. Please validate your network connection and verify if Docker daemon is running properly."
)


# HELM ERRORS
HELM_COMMAND_ERROR = ErrorClass(
    "HELM_COMMAND_ERROR",
    "Please verify if Helm is installed."
)

HELM_VERSION_ERROR = ErrorClass(
    "HELM_VERSION_ERROR",
    "An error occurred while retrieving Helm version. Please make sure that you have the latest Azure CLI version, and that you are using the recommended Helm version."
)


# NOTARY ERRORS
NOTARY_COMMAND_ERROR = ErrorClass(
    "NOTARY_COMMAND_ERROR",
    "Please verify if notary is installed."
)

NOTARY_VERSION_ERROR = ErrorClass(
    "NOTARY_VERSION_ERROR",
    "An error occurred while retrieving notary version. Please make sure that you have the latest Azure CLI version, and that you are using the recommended notary version."
)


# CONNECTIVITY ERRORS
CONNECTIVITY_DNS_ERROR = ErrorClass(
    "CONNECTIVITY_DNS_ERROR",
    "Failed to reach DNS for registry '{}'. Please check if the spelling is correct, if the CLI environment is on correct cloud and your network connectivity."
)


CONNECTIVITY_FORBIDDEN_ERROR = ErrorClass(
    "CONNECTIVITY_FORBIDDEN_ERROR",
    "Looks like you don't have access to registry '{}'. To see configured firewall rules, run 'az acr show --query networkRuleSet --name {}'. Details: https://docs.microsoft.com/azure/container-registry/container-registry-health-error-reference#connectivity_forbidden_error"
)


CONNECTIVITY_CHALLENGE_ERROR = ErrorClass(
    "CONNECTIVITY_CHALLENGE_ERROR",
    "Registry '{}' did not issue a challenge."
)


CONNECTIVITY_AAD_LOGIN_ERROR = ErrorClass(
    "CONNECTIVITY_AAD_LOGIN_ERROR",
    "Registry '{}' does not support AAD login."
)


CONNECTIVITY_WWW_AUTHENTICATE_ERROR = ErrorClass(
    "CONNECTIVITY_WWW_AUTHENTICATE_ERROR",
    "Registry '{}' did not ask properly for a basic or a bearer authentication."
)


CONNECTIVITY_REFRESH_TOKEN_ERROR = ErrorClass(
    "CONNECTIVITY_REFRESH_TOKEN_ERROR",
    "Access to registry '{}' was denied. Response code: {}. Please try running 'az login' again to refresh permissions."
)


CONNECTIVITY_ACCESS_TOKEN_ERROR = ErrorClass(
    "CONNECTIVITY_ACCESS_TOKEN_ERROR",
    "Access to registry '{}' was denied. Response code: {}. Please try running 'az login' again to refresh permissions."
)


CONNECTIVITY_SSL_ERROR = ErrorClass(
    "CONNECTIVITY_SSL_ERROR",
    "Unable to establish a secure connection to the registry '{}'. Are you using a proxy server?"
)


# GENERAL ERRORS
LOGIN_SERVER_ERROR = ErrorClass(
    "LOGIN_SERVER_ERROR",
    "An error occurred while retrieving the login server for registry '{}'. Please check if the spelling is correct, if the CLI environment is on correct cloud and if you have the right permissions on it."
)


UNEXPECTED_ERROR = ErrorClass(
    "UNEXPECTED_ERROR",
    "An unexpected error occurred."
)

CMK_MANAGED_IDENTITY_ERROR = ErrorClass(
    "CMK_ERROR",
    "The identity used for registry '{}' encryption doesn't exist."
)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def backup_exception_handler(ex):
    from azure.core.exceptions import HttpResponseError
    if isinstance(ex, HttpResponseError) and ex.message:
        raise HttpResponseError(ex.message)
    raise ex



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from datetime import datetime
from azure.cli.core.azclierror import RequiredArgumentMissingError, MutuallyExclusiveArgumentError, \
    ArgumentUsageError, InvalidArgumentValueError
from azure.mgmt.recoveryservicesbackup.activestamp.models import StorageType
# Argument types


def datetime_type(string):
    """ Validate UTC datettime in accepted format. Examples: 31-12-2017, 31-12-2017-05:30:00 """
    accepted_date_formats = ['%d-%m-%Y', '%d-%m-%Y-%H:%M:%S']
    for form in accepted_date_formats:
        try:
            return datetime.strptime(string, form)
        except ValueError:  # checks next format
            pass
    raise InvalidArgumentValueError("""
    Input '{}' is not valid. Valid example: 31-12-2017, 31-12-2017-05:30:00
    """.format(string))


def validate_mi_used_for_restore_disks(vault_identity, use_system_assigned_msi, identity_id):
    if (use_system_assigned_msi or identity_id) and vault_identity is None:
        raise ArgumentUsageError("Please ensure that Selected MI is enabled for the vault")
    if use_system_assigned_msi:
        if vault_identity.type is None or "systemassigned" not in vault_identity.type.lower():
            raise ArgumentUsageError("Please ensure that System MI is enabled for the vault")
    if identity_id:
        if vault_identity.type is not None and "userassigned" in vault_identity.type.lower():
            if identity_id.lower() not in (id.lower() for id in vault_identity.user_assigned_identities.keys()):
                raise ArgumentUsageError("""
                Vault does not have the specified User MI.
                Please ensure you've provided the correct --mi-user-assigned.
                """)
        else:
            raise ArgumentUsageError("Please ensure that User MI is enabled for the vault")


def validate_wl_restore(item, item_type, restore_mode, recovery_mode):
    # if source_resource_id is None or source_resource_id.lower() != item.properties.source_resource_id.lower():
    #    raise InvalidArgumentValueError("""
    #        The source_resource_id specified in recovery config file is incorrect. Please correct it and retry the
    #        operation. Correct value should be - {}.
    #        """.format(item.properties.source_resource_id))

    # if workload_type is None or workload_type.lower() != item.properties.workload_type.lower():
    #    raise InvalidArgumentValueError("""
    #        The workload_type specified in recovery config file is incorrect. Please correct it and retry the
    #        operation. Correct value should be - {}.
    #        """.format(item.properties.workload_type))

    if item_type is None or item_type.lower() not in ['sql', 'saphana']:
        raise InvalidArgumentValueError("""
            The item_type specified in recovery config file is incorrect. Please correct it and retry the
            operation. Allowed values are: 'SQL', 'SAPHana'.
            """)

    if item_type.lower() not in item.properties.workload_type.lower():
        raise InvalidArgumentValueError("""
            The item_type and workload_type specified in recovery config file does not match. Please correct either
            of them and retry the operation.
            """)

    if restore_mode not in ['OriginalLocation', 'AlternateLocation']:
        raise InvalidArgumentValueError("""
            The restore_mode specified in recovery config file is incorrect. Please correct it and retry the
            operation. Allowed values are: 'OriginalLocation', 'AlternateLocation'.
            """)

    if recovery_mode is not None and recovery_mode != 'FileRecovery':
        raise InvalidArgumentValueError("""
            The recovery_mode specified in recovery config file is incorrect. Please correct it and retry the
            operation.
            """)


def validate_log_point_in_time(log_point_in_time, time_range_list):
    for time_range in time_range_list:
        if (time_range.start_time.replace(tzinfo=None) <= log_point_in_time <=
                time_range.end_time.replace(tzinfo=None)):
            return
    raise InvalidArgumentValueError("""
        The log point in time specified in recovery config file does not belong to the allowed time range.
        Please correct it and retry the operation. To check the permissible time range use:
        'az backup recoverypoint show-log-chain' command.
        """)


def validate_crr(target_rg_id, rehydration_priority):
    if target_rg_id is None:
        raise RequiredArgumentMissingError("Please provide target resource group using --target-resource-group.")

    if rehydration_priority is not None:
        raise MutuallyExclusiveArgumentError("Archive restore isn't supported for secondary region.")


def validate_czr(backup_config_response, recovery_point):
    backup_storage_redundancy = backup_config_response.properties.storage_type
    cross_region_restore_flag = backup_config_response.properties.cross_region_restore_flag
    if (cross_region_restore_flag or backup_storage_redundancy == StorageType.ZONE_REDUNDANT):
        if recovery_point.tier_type is not None and recovery_point.tier_type == "VaultStandard":
            if recovery_point.properties.zones is None:
                raise ArgumentUsageError("""
                Please ensure that the recovery point is zone pinned or remove --target-zone argument.
                """)
        else:
            raise ArgumentUsageError("""
            Please ensure that the given RP tier type is 'VaultStandard' or remove --target-zone argument.
            """)
    else:
        raise ArgumentUsageError("""
        Please ensure either the vault storage redundancy is ZoneRedundant or the vault has CRR enabled or try
        removing --target-zone argument.
        """)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
from azure.cli.core.commands.parameters import get_enum_type, get_three_state_flag


def load_arguments(self, _):

    with self.argument_context('configure') as c:
        c.argument('defaults', nargs='+', options_list=('--defaults', '-d'))
        c.argument('list_defaults', options_list=('--list-defaults', '-l'),
                   arg_type=get_three_state_flag(), help='list all applicable defaults')
        c.argument('scope', arg_type=get_enum_type(['global', 'local']), default='global',
                   help='scope of defaults. Using "local" for settings only effective under current folder')
        c.ignore('_subscription')  # ignore the global subscription param

    with self.argument_context('cache') as c:
        c.argument('resource_type', options_list=['--resource-type', '-t'], help='The resource type.')
        c.argument('item_name', options_list=['--name', '-n'], help='The resource name.')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps

helps['identity'] = """
type: group
short-summary: Managed Identities
"""

helps['identity create'] = """
type: command
short-summary: Create Identities
examples:
  - name: Create an identity.
    text: |
        az identity create --name MyIdentity --resource-group MyResourceGroup
"""

helps['identity list'] = """
type: command
short-summary: List Managed Identities
"""

helps['identity list-operations'] = """
type: command
short-summary: Lists available operations for the Managed Identity provider
"""
